#ifndef PARSER_H
#define PARSER_H

#include <json/json.h>
#include <map>
#include <vector>
#include <string>
#include <iostream>
#include <stdexcept>
#include "item.h"

using namespace std;
using namespace item;

namespace parser {
    class Parser {
    public:
        Parser(char *filename, map<string, Item *> &all, vector<InputItem *> &in, vector<OutputItem *> &out, vector<ConnectedItem *> &logic) {
            json_object *scheme = json_object_from_file(filename);
            if(scheme == (struct json_object *)error_ptr(-1))
                throw logic_error("can't read input file");

            json_object *alu_scheme = json_object_object_get(scheme, "ALU");
            if(alu_scheme == NULL)
                throw logic_error("can't find object ALU");

            int arrLen = json_object_array_length(alu_scheme);
            for(int i = 0; i < arrLen; i++) {
                json_object *element = json_object_array_get_idx(alu_scheme, i);
                json_object *element_id = json_object_object_get(element, "id");
                string e_id(json_object_get_string(element_id));
                if(all.find(e_id) != all.end())
                    throw logic_error("redefinition");

                json_object *element_type = json_object_object_get(element, "type");
                string e_type(json_object_get_string(element_type));
                if(e_type == "in") {
                    InputItem *tmp = new InputItem(e_id);
                    all[e_id] = tmp;
                    in.push_back(tmp);
                }
                else if(e_type == "out") {
                    OutputItem *tmp = new OutputItem(e_id);
                    all[e_id] = tmp;
                    out.push_back(tmp);
                }
                else if(e_type == "multiplexer") {
                    MultiplexerItem *tmp = new MultiplexerItem(e_id);
                    all[e_id] = tmp;
                    logic.push_back(tmp);
                }
                else if(e_type == "xor" || e_type == "or" || e_type == "and") {
                    LogicItem *tmp = new LogicItem(e_id);
                    all[e_id] = tmp;
                    logic.push_back(tmp);
                }
                else 
                    throw logic_error("unkonwn type");
            }
        }
    };
}

#endif
